@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-rss-extension.c,v 1.15 2006/09/13 19:01:27 chpe Exp $
+ *  $Id: ephy-rss-extension.c 1432 2007-01-09 10:38:37Z lferrett $
  */
 
 #include "config.h"
@@ -38,7 +38,6 @@
 #include <gtk/gtkuimanager.h>
 
 #include <gtk/gtkeventbox.h>
-#include <gtk/gtkframe.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkimage.h>
 
@@ -64,13 +63,7 @@ static GType type = 0;
 #define MENU_PATH	"/menubar/ToolsMenu"
 
 /* Status Bar stuff */
-
-/* Test these
-	"stock_hyperlink-toolbar"	<-- Globe with a link
-	"stock_connect-to-url"		<-- Globe with a plug
-	"stock_internet"			<-- Simple globe
-*/
-#define STOCK_ICON	"stock_hyperlink-toolbar"
+#define FEED_ICON	"feed-presence"
 
 static void ephy_rss_display_cb 	(GtkAction *action, 
 					 EphyWindow *window);
@@ -87,7 +80,6 @@ typedef struct
 	GtkAction *subscribe_action;
 	guint ui_id;
 
-	GtkWidget *frame;
 	GtkWidget *evbox;
 } WindowData;
 
@@ -262,9 +254,8 @@ ephy_rss_update_statusbar (EphyWindow *window,
 	/* Show / Hide statusbar icon */
 	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
 	g_return_if_fail (data != NULL);
-	g_return_if_fail (data->frame != NULL);
 	
-	g_object_set (data->frame, "visible", show, NULL);
+	g_object_set (data->evbox, "visible", show, NULL);
 }
 
 /* Update the menu item availability */
@@ -381,15 +372,10 @@ ephy_rss_create_statusbar_icon (EphyWindow *window,
 	statusbar = EPHY_STATUSBAR (ephy_window_get_statusbar (window));
 	g_return_if_fail (statusbar != NULL);
 
-	data->frame = gtk_frame_new (NULL);
-	/* don't show the frame */
-
 	data->evbox = gtk_event_box_new ();
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (data->evbox), FALSE);
-	gtk_container_add (GTK_CONTAINER (data->frame), data->evbox);
-	gtk_widget_show (data->evbox);
 
-	icon = gtk_image_new_from_icon_name (STOCK_ICON, GTK_ICON_SIZE_MENU);
+	icon = gtk_image_new_from_icon_name (FEED_ICON, GTK_ICON_SIZE_MENU);
 	gtk_container_add (GTK_CONTAINER (data->evbox), icon);
 	gtk_widget_show (icon);
 
@@ -397,7 +383,7 @@ ephy_rss_create_statusbar_icon (EphyWindow *window,
 	gtk_tooltips_set_tip (statusbar->tooltips, data->evbox, tooltip, NULL);
 	g_free (tooltip);
 
-	ephy_statusbar_add_widget (statusbar, data->frame);
+	ephy_statusbar_add_widget (statusbar, data->evbox);
 
 	g_signal_connect_after (data->evbox, "button-press-event",
 				G_CALLBACK (ephy_rss_statusbar_icon_clicked_cb),
@@ -413,12 +399,11 @@ ephy_rss_destroy_statusbar_icon (EphyWindow *window,
 	statusbar = EPHY_STATUSBAR (ephy_window_get_statusbar (window));
 	g_return_if_fail (statusbar != NULL);
 
-	g_return_if_fail (data->frame != NULL);
 	g_return_if_fail (data->evbox != NULL);	
 
 	gtk_tooltips_set_tip (statusbar->tooltips, GTK_WIDGET (data->evbox), NULL, NULL);
 
-	ephy_statusbar_remove_widget (statusbar, GTK_WIDGET (data->frame));
+	ephy_statusbar_remove_widget (statusbar, GTK_WIDGET (data->evbox));
 }
 
 /* Create the menu item to subscribe to an rss feed */
